@@ -3,11 +3,16 @@ mod preserves;
 mod subcommands;
 
 use clap::{App, AppSettings, Arg, crate_authors, crate_version};
-use preserves::{Check, Summary, CheckList, datasource::{STATUS, REFS, REMOTE}};
+use preserves::{Check, Summary, CheckList, datasource::Group};
 use tera::{Tera, Context};
 use lazy_static::lazy_static;
 use include_dir::{include_dir,Dir,DirEntry};
 use std::path::Path;
+use git::{LsRemote, GetStatus, ForEachRef};
+use fake::{Fake, Faker};
+use rand::{Rng,rngs::StdRng};
+use rand::SeedableRng;
+use getrandom::getrandom;
 
 lazy_static! {
   pub static ref TEMPLATES: Dir<'static> = include_dir!("src/templates");
@@ -72,6 +77,28 @@ fn main() -> ! {
       .help("provide a template source directory")
       .takes_value(true)
     )
+    .arg(
+      Arg::with_name("json")
+      .long("json")
+      .short("j")
+      .help("emits json rather than human readable report")
+      .conflicts_with("format")
+      .conflicts_with("template")
+      .conflicts_with("quiet")
+    )
+    .arg(
+      Arg::with_name("example")
+      .long("example")
+      .help("generates example output for template development")
+      .conflicts_with("checks")
+    )
+    .arg(
+      Arg::with_name("seed-file")
+      .long("seed-file")
+      .help("reads (or writes) the example seed from (or to) a file")
+      .requires("example")
+      .takes_value(true)
+    )
     .arg(
       Arg::with_name("checks")
       .long("checks")
@@ -105,60 +132,65 @@ fn main() -> ! {
       println!("Required sources: {:?}", reqs)
     }
 
-    let ls_remote = if reqs.includes(REMOTE) {
-      git::ls_remote().unwrap_or_else(&error_status(128))
+    let summary = if opt.is_present("example") {
+      let mut r = load_rng(opt.value_of("seed-file"));
+      Summary::new(
+        (Faker, 0..10).fake_with_rng(&mut r),
+        Faker.fake_with_rng(&mut r),
+        (Faker, 0..10).fake_with_rng(&mut r),
+        Check::all_checks()
+      )
     } else {
-      vec![]
-    };
-
-    let status = if reqs.includes(STATUS) {
-      git::status().unwrap_or_else(&error_status(129))
-    } else {
-      git::Status::default()
-    };
-
-    let for_each_ref = if reqs.includes(REFS) {
-      git::for_each_ref().unwrap_or_else(&error_status(130))
-    } else {
-      vec![]
+      Summary::new(
+        collect(LsRemote, reqs, 128),
+        collect(GetStatus, reqs, 129),
+        collect(ForEachRef, reqs, 130),
+        checks
+      )
     };
 
     if opt.is_present("debug") {
-      println!("{:#?}\n{:#?}\n{:#?}", status, for_each_ref, ls_remote);
+      println!("{:#?}\n{:#?}\n{:#?}", summary.status, summary.for_each_ref, summary.ls_remote);
     }
 
-    let summary = Summary::new(ls_remote, status, for_each_ref, checks);
-
     if opt.is_present("debug") {
       println!("will exit: {}", summary.exit_status())
     }
 
     if !opt.is_present("quiet") {
-      //println!("status: {}", serde_json::to_string(&summary.status)?);
-      //println!("items: {}", serde_json::to_string(&summary.items())?);
-      let mut context = Context::default();
-      context.insert("items", &summary.items());
-      context.insert("status", &summary.status);
-      let body = if let Some(tdir) = opt.value_of("template") {
-        let tpath = Path::new(tdir).join("**");
-        let t = Tera::new(
-          tpath.to_str()
-          .ok_or("couldn't convert path to utf8")
-          .unwrap_or_else(&error_status(133))
-        ).unwrap_or_else(&error_status(132));
-        t.render(opt.value_of("format").expect("format has no value"), &context)
-          .unwrap_or_else(&error_status(131))
+        let mut context = Context::default();
+        context.insert("items", &summary.items());
+        context.insert("status", &summary.status);
+      if opt.is_present("json") {
+        println!("{}", context.into_json());
       } else {
-        TMPL.render(opt.value_of("format").expect("format has no value"), &context)
-          .unwrap_or_else(&error_status(131))
-      };
+        //println!("status: {}", serde_json::to_string(&summary.status)?);
+        //println!("items: {}", serde_json::to_string(&summary.items())?);
+        let body = if let Some(tdir) = opt.value_of("template") {
+          let tpath = Path::new(tdir).join("**");
+          let t = Tera::new(
+            tpath.to_str()
+            .ok_or("couldn't convert path to utf8")
+            .unwrap_or_else(&error_status(133))
+          ).unwrap_or_else(&error_status(132));
+          t.render(opt.value_of("format").expect("format has no value"), &context)
+            .unwrap_or_else(&error_status(131))
+        } else {
+          TMPL.render(opt.value_of("format").expect("format has no value"), &context)
+            .unwrap_or_else(&error_status(131))
+        };
 
-      print!("{}", body);
+        print!("{}", body);
+      }
     }
 
     std::process::exit(summary.exit_status())
 }
 
+fn collect<T>( provider: impl git::Provider<Data = T>, reqs: Group, errcode: i32,) -> T {
+  provider.collect(reqs).unwrap_or_else(&error_status(errcode))
+}
+
 fn error_status<T, E: core::fmt::Debug>(n: i32) -> impl Fn(E) -> T {
   return move |e: E| {
     println!("{:?}", e);
@@ -166,6 +198,27 @@ fn error_status<T, E: core::fmt::Debug>(n: i32) -> impl Fn(E) -> T {
   }
 }
 
+fn load_rng(seedpath: Option<&str>) -> impl Rng {
+  let ref mut seed = [0; 32];
+  use std::fs::File;
+  use std::io::{Read,Write};
+  match seedpath {
+    None => getrandom(seed).unwrap_or_else(&error_status(131)),
+    Some(path) => {
+      match File::open(path) {
+        Ok(mut f) => f.read(seed).unwrap_or_else(&error_status(132)),
+        Err(_) => {
+          getrandom(seed).unwrap_or_else(&error_status(133));
+          let mut f = File::create(path).unwrap_or_else(&error_status(134));
+          f.write(seed).unwrap_or_else(&error_status(135))
+        }
+      };
+    }
+  }
+
+  StdRng::from_seed(*seed)
+}
+
 /* Stages of execution:
  * Choose Checks
  * Hassle Git for data
